# Generated by Django 4.2.7 on 2023-12-04 22:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_adherence", "0003_nonadherencereasons_extra_value"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="nonadherencereasons",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Non-Adherence Reasons",
                "verbose_name_plural": "Non-Adherence Reasons",
            },
        ),
        migrations.RemoveIndex(
            model_name="nonadherencereasons",
            name="edc_adheren_id_0617c0_idx",
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AddIndex(
            model_name="nonadherencereasons",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_adheren_display_4c242c_idx",
            ),
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-12 22:37

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_action_item",
            "0036_alter_actionitem_priority_alter_actionitem_site_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="actionitem",
            name="reference_model",
        ),
        migrations.RemoveField(
            model_name="actionitem",
            name="related_reference_model",
        ),
        migrations.RemoveField(
            model_name="historicalactionitem",
            name="reference_model",
        ),
        migrations.RemoveField(
            model_name="historicalactionitem",
            name="related_reference_model",
        ),
    ]



from .utils import get_adverse_event_app_label, get_hospitalization_model_app_label

ae_codenames = [
    "edc_adverse_event.nav_ae_section",
    "edc_adverse_event.view_ae_listboard",
    "edc_adverse_event.view_aeclassification",
    "edc_adverse_event.view_causeofdeath",
    "edc_adverse_event.view_saereason",
    f"{get_adverse_event_app_label()}.add_aefollowup",
    f"{get_adverse_event_app_label()}.add_aeinitial",
    f"{get_adverse_event_app_label()}.add_aesusar",
    f"{get_adverse_event_app_label()}.add_deathreport",
    f"{get_hospitalization_model_app_label()}.add_hospitalization",
    f"{get_adverse_event_app_label()}.change_aefollowup",
    f"{get_adverse_event_app_label()}.change_aeinitial",
    f"{get_adverse_event_app_label()}.change_aesusar",
    f"{get_adverse_event_app_label()}.change_deathreport",
    f"{get_hospitalization_model_app_label()}.change_hospitalization",
    f"{get_adverse_event_app_label()}.delete_aefollowup",
    f"{get_adverse_event_app_label()}.delete_aeinitial",
    f"{get_adverse_event_app_label()}.delete_aesusar",
    f"{get_adverse_event_app_label()}.delete_deathreport",
    f"{get_hospitalization_model_app_label()}.delete_hospitalization",
    f"{get_adverse_event_app_label()}.view_aefollowup",
    f"{get_adverse_event_app_label()}.view_aeinitial",
    f"{get_adverse_event_app_label()}.view_aesusar",
    f"{get_adverse_event_app_label()}.view_aetmg",
    f"{get_adverse_event_app_label()}.view_deathreport",
    f"{get_adverse_event_app_label()}.view_deathreporttmg",
    f"{get_adverse_event_app_label()}.view_deathreporttmgsecond",
    f"{get_hospitalization_model_app_label()}.view_hospitalization",
    f"{get_adverse_event_app_label()}.view_historicalaefollowup",
    f"{get_adverse_event_app_label()}.view_historicalaeinitial",
    f"{get_adverse_event_app_label()}.view_historicalaesusar",
    f"{get_adverse_event_app_label()}.view_historicalaetmg",
    f"{get_adverse_event_app_label()}.view_historicaldeathreport",
    f"{get_adverse_event_app_label()}.view_historicaldeathreporttmg",
    f"{get_adverse_event_app_label()}.view_historicaldeathreporttmgsecond",
    f"{get_hospitalization_model_app_label()}.view_historicalhospitalization",
]

tmg_codenames = [
    "edc_adverse_event.nav_tmg_section",
    "edc_adverse_event.view_tmg_listboard",
    "edc_adverse_event.view_aeclassification",
    "edc_adverse_event.view_causeofdeath",
    "edc_adverse_event.view_saereason",
    f"{get_adverse_event_app_label()}.add_aetmg",
    f"{get_adverse_event_app_label()}.add_deathreporttmg",
    f"{get_adverse_event_app_label()}.add_deathreporttmgsecond",
    f"{get_adverse_event_app_label()}.change_aetmg",
    f"{get_adverse_event_app_label()}.change_deathreporttmg",
    f"{get_adverse_event_app_label()}.change_deathreporttmgsecond",
    f"{get_adverse_event_app_label()}.delete_aetmg",
    f"{get_adverse_event_app_label()}.delete_deathreporttmg",
    f"{get_adverse_event_app_label()}.delete_deathreporttmgsecond",
    f"{get_adverse_event_app_label()}.view_aefollowup",
    f"{get_adverse_event_app_label()}.view_aeinitial",
    f"{get_adverse_event_app_label()}.view_aesusar",
    f"{get_adverse_event_app_label()}.view_aetmg",
    f"{get_adverse_event_app_label()}.view_deathreport",
    f"{get_adverse_event_app_label()}.view_deathreporttmg",
    f"{get_adverse_event_app_label()}.view_deathreporttmgsecond",
    f"{get_hospitalization_model_app_label()}.view_hospitalization",
    f"{get_adverse_event_app_label()}.view_historicalaefollowup",
    f"{get_adverse_event_app_label()}.view_historicalaeinitial",
    f"{get_adverse_event_app_label()}.view_historicalaesusar",
    f"{get_adverse_event_app_label()}.view_historicalaetmg",
    f"{get_adverse_event_app_label()}.view_historicaldeathreport",
    f"{get_adverse_event_app_label()}.view_historicaldeathreporttmg",
    f"{get_adverse_event_app_label()}.view_historicaldeathreporttmgsecond",
    f"{get_hospitalization_model_app_label()}.view_historicalhospitalization",
]

tmg_view_codenames = [c for c in tmg_codenames if "view_" in c]

ae_dashboard_tuples = (("edc_adverse_event.view_ae_listboard", "Can view AE listboard"),)
ae_navbar_tuples = (("edc_adverse_event.nav_ae_section", "Can view AE section"),)

tmg_dashboard_tuples = (("edc_adverse_event.view_tmg_listboard", "Can view TMG Listboard"),)
tmg_navbar_tuples = (("edc_adverse_event.nav_tmg_section", "Can view TMG section"),)



from django.core.checks import Warning

from .site_action_items import site_action_items


def edc_action_item_checks(app_configs, **kwargs):
    errors = []
    for name, action_cls in site_action_items.registry.items():
        try:
            action_cls.reference_model_cls().history
        except AttributeError as e:
            if "history" not in str(e):
                raise
            errors.append(
                Warning(
                    (
                        f"Reference model used by action mcs {action_cls} "
                        f"has no history manager."
                    ),
                    hint="History manager is need to detect changes.",
                    obj=action_cls,
                    id="edc_action_item.W001",
                )
            )
    return errors



from .export_tracking_fields_model_mixin import ExportTrackingFieldsModelMixin

# from .notification_model_mixin import NotificationMixin
# from .notification_plan_model_mixin import NotificationPlanMixin



from django import forms

from ...models import StockTransfer, StockTransferItem


class StockTransferForm(forms.ModelForm):

    def clean(self):
        cleaned_data = super().clean()
        items_qs = StockTransferItem.objects.filter(stock_transfer__pk=self.instance.pk)
        if cleaned_data.get("to_location") and items_qs.count() > 0:
            if (
                items_qs[0].stock.allocation.registered_subject.site
                != cleaned_data.get("to_location").site
            ):
                raise forms.ValidationError(
                    {
                        "to_location": (
                            "Invalid location. Does not match the intended location of "
                            "existing stock items for this transfer."
                        )
                    }
                )
        return cleaned_data

    class Meta:
        model = StockTransfer
        fields = "__all__"
        help_text = {"transfer_identifier": "(read-only)"}
        widgets = {
            "transfer_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from dateutil.relativedelta import relativedelta
from django.utils.translation import gettext as _

from edc_utils import formatted_date
from edc_utils.date import floor_secs, to_local
from edc_visit_schedule.exceptions import (
    ScheduledVisitWindowError,
    UnScheduledVisitWindowError,
)
from edc_visit_schedule.utils import get_lower_datetime

from ..constants import COMPLETE_APPT, INCOMPLETE_APPT

if TYPE_CHECKING:
    from edc_appointment.models import Appointment

UNSCHEDULED_WINDOW_ERROR = "unscheduled_window_error"
SCHEDULED_WINDOW_ERROR = "scheduled_window_error"


class WindowPeriodFormValidatorMixin:
    def validate_appt_datetime_in_window_period(self, appointment: Appointment, *args) -> None:
        self.datetime_in_window_or_raise(appointment, *args)

    @staticmethod
    def ignore_window_period_for_unscheduled(
        appointment: Appointment, proposed_appt_datetime: datetime
    ) -> bool:
        """Returns True if this is an unscheduled appt"""
        value = False
        if (
            appointment
            and appointment.visit_code_sequence > 0
            and appointment.next
            and appointment.next.appt_status in [INCOMPLETE_APPT, COMPLETE_APPT]
            and to_local(proposed_appt_datetime) < to_local(appointment.next.appt_datetime)
        ):
            value = True
        return value

    def datetime_in_window_or_raise(
        self,
        appointment: Appointment,
        proposed_appt_datetime: datetime,
        form_field: str,
    ):
        if proposed_appt_datetime:
            try:
                appointment.schedule.datetime_in_window(
                    timepoint_datetime=to_local(appointment.timepoint_datetime),
                    dt=to_local(proposed_appt_datetime),
                    visit_code=appointment.visit_code,
                    visit_code_sequence=appointment.visit_code_sequence,
                    baseline_timepoint_datetime=to_local(
                        self.baseline_timepoint_datetime(appointment)
                    ),
                )
            except UnScheduledVisitWindowError:
                if not self.ignore_window_period_for_unscheduled(
                    appointment, proposed_appt_datetime
                ):
                    # TODO: fix the dates on this message to match e.message
                    lower = floor_secs(get_lower_datetime(appointment))
                    try:
                        # one day less than the next related_visit, if it exists
                        upper = floor_secs(
                            appointment.next.related_visit.report_datetime
                            - relativedelta(days=1)
                        )
                    except AttributeError:
                        # lower bound of next appointment
                        upper = floor_secs(get_lower_datetime(appointment.next))
                    dt_lower = formatted_date(to_local(lower))
                    dt_upper = formatted_date(to_local(upper))
                    self.raise_validation_error(
                        {
                            form_field: (
                                _(
                                    "Invalid. Expected a date between "
                                    "%(dt_lower)s and %(dt_upper)s (U)."
                                )
                                % dict(dt_lower=dt_lower, dt_upper=dt_upper)
                            )
                        },
                        UNSCHEDULED_WINDOW_ERROR,
                    )
            except ScheduledVisitWindowError as e:
                self.raise_validation_error({form_field: (str(e))}, SCHEDULED_WINDOW_ERROR)

    @staticmethod
    def baseline_timepoint_datetime(appointment: Appointment) -> datetime:
        return appointment.__class__.objects.first_appointment(
            subject_identifier=appointment.subject_identifier,
            visit_schedule_name=appointment.visit_schedule_name,
            schedule_name=appointment.schedule_name,
        ).timepoint_datetime



from __future__ import annotations

from django.db import models


class CharField2(models.CharField):
    def __init__(self, *args, metadata: str | dict | None = None, **kwargs):
        self.metadata = metadata
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.metadata is not None:
            kwargs["metadata"] = self.metadata
        return name, path, args, kwargs


class IntegerField2(models.IntegerField):
    def __init__(self, *args, metadata: str | dict | None = None, **kwargs):
        self.metadata = metadata
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.metadata is not None:
            kwargs["metadata"] = self.metadata
        return name, path, args, kwargs


class ManyToManyField2(models.ManyToManyField):
    def __init__(self, *args, metadata: str | dict | None = None, **kwargs):
        self.metadata = metadata
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if self.metadata is not None:
            kwargs["metadata"] = self.metadata
        return name, path, args, kwargs



from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_audit_fields.admin import audit_fieldset_tuple

from edc_model_admin.history import SimpleHistoryAdmin
from edc_utils.date import to_local

from ...admin_site import edc_pharmacy_admin
from ...auth_objects import PHARMACY_SUPER_ROLE
from ...forms import OrderForm
from ...forms.stock import OrderFormSuper
from ...models import Order, OrderItem, Receive
from ..model_admin_mixin import ModelAdminMixin


@admin.register(Order, site=edc_pharmacy_admin)
class OrderAdmin(ModelAdminMixin, SimpleHistoryAdmin):
    change_list_title = "Pharmacy: Orders"
    change_form_title = "Pharmacy: Order"
    history_list_display = ()
    show_object_tools = True
    show_cancel = True
    list_per_page = 20

    form = OrderForm
    insert_before_fieldset = "Audit"
    ordering = ("-order_identifier",)

    fieldsets = (
        (
            None,
            {"fields": (["order_identifier", "order_datetime", "supplier", "title"])},
        ),
        ("Quantity", {"fields": (["item_count"])}),
        ("Status", {"fields": (["sent", "status"])}),
        ("Section C: Comment / Notes", {"fields": ("comment",)}),
        audit_fieldset_tuple,
    )

    list_display = (
        "identifier",
        "supplier",
        "short_title",
        "order_date",
        "status",
        "qty",
        "items",
        "receive_changelist",
        "created",
        "modified",
    )
    list_filter = ("sent", "status", "order_datetime")
    radio_fields = {"status": admin.VERTICAL}
    search_fields = ("id", "order_identifier", "title")
    readonly_fields = ("order_identifier", "sent")

    @admin.display(description="ORDER #", ordering="-order_identifier")
    def identifier(self, obj):
        return obj.order_identifier

    @admin.display(description="Title", ordering="title")
    def short_title(self, obj):
        return format_html(
            '<span title="{title}">{short_title}</span>',
            title=obj.title,
            short_title=obj.title[:20],
        )

    @admin.display(description="QTY", ordering="-item_count")
    def qty(self, obj):
        return obj.item_count

    @admin.display(description="Order items", ordering="-order_identifier")
    def items(self, obj):
        add_order_items_button = ""
        if obj.item_count > OrderItem.objects.filter(order=obj).count():
            url = reverse("edc_pharmacy_admin:edc_pharmacy_orderitem_add")
            next_url = "edc_pharmacy_admin:edc_pharmacy_order_changelist"
            url = f"{url}?next={next_url}&order={obj.id!s}&q={obj.order_identifier!s}"
            context = dict(url=url, label="Add order item")
            add_order_items_button = render_to_string(
                "edc_pharmacy/stock/items_as_button.html", context=context
            )
        count = OrderItem.objects.filter(order=obj).count()
        url = reverse("edc_pharmacy_admin:edc_pharmacy_orderitem_changelist")
        url = f"{url}?q={obj.order_identifier}"
        context = dict(
            url=url, label=f"Ordered items ({count})", title="Go to items on this order"
        )
        order_items_link = render_to_string(
            "edc_pharmacy/stock/items_as_link.html", context=context
        )
        rendered = [add_order_items_button, order_items_link]
        return format_html(
            "{}",
            mark_safe("<BR>".join([r for r in rendered if r])),  # nosec B703, B308
        )

    @admin.display(description="Order date", ordering="order_datetime")
    def order_date(self, obj):
        return to_local(obj.order_datetime).date()

    @admin.display(description="Receive #")
    def receive_changelist(self, obj):
        try:
            rcv_obj = Receive.objects.get(order=obj)
        except ObjectDoesNotExist:
            pass
        else:
            url = reverse("edc_pharmacy_admin:edc_pharmacy_receive_changelist")
            url = f"{url}?q={rcv_obj.receive_identifier!s}"
            context = dict(url=url, label=rcv_obj.receive_identifier, title="Receive #")
            return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)
        return None

    def get_form(self, request, obj=None, **kwargs):
        try:
            request.user.userprofile.roles.get(name=PHARMACY_SUPER_ROLE)
        except ObjectDoesNotExist:
            pass
        else:
            self.form = OrderFormSuper
        return super().get_form(request, obj, **kwargs)



import django.utils.timezone
from django.db import models

from edc_constants.constants import NOT_APPLICABLE
from edc_model.validators import date_not_future
from edc_model_fields.fields.other_charfield import OtherCharField

from ...choices import AE_GRADE, STUDY_DRUG_RELATIONSHIP
from ...models import AeClassification


class AeInitialFieldsModelMixin(models.Model):
    ae_auto_created = models.BooleanField(max_length=25, default=False, editable=False)

    ae_auto_created_criteria = models.CharField(
        max_length=50, default=NOT_APPLICABLE, editable=False
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=django.utils.timezone.now
    )

    ae_classification = models.ForeignKey(
        AeClassification,
        on_delete=models.PROTECT,
        verbose_name="Adverse Event (AE) Classification",
        null=True,
        blank=False,
    )

    ae_classification_other = OtherCharField(max_length=250, blank=True, null=True)

    ae_description = models.TextField(verbose_name="Adverse Event (AE) description")

    ae_awareness_date = models.DateField(
        verbose_name="AE Awareness date",
        default=django.utils.timezone.now,
        validators=[date_not_future],
    )

    ae_start_date = models.DateField(
        verbose_name="Actual Start Date of AE",
        default=django.utils.timezone.now,
        validators=[date_not_future],
    )

    ae_grade = models.CharField(verbose_name="Severity of AE", max_length=25, choices=AE_GRADE)

    study_drug_relation = models.CharField(  # noqa: DJ001
        verbose_name="Relation to study drug:",
        max_length=25,
        choices=STUDY_DRUG_RELATIONSHIP,
        null=True,
    )

    class Meta:
        abstract = True



from edc_identifier.simple_identifier import SimpleUniqueIdentifier


class ActionIdentifier(SimpleUniqueIdentifier):
    random_string_length = 12
    identifier_type = "action_identifier"
    identifier_prefix = "AC"
    make_human_readable = True
